package de.pzzz.vertx.process;

import java.io.Serializable;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import de.pzzz.vertx.Startup;
import de.pzzz.vertx.worker.DeployableWorker;
import de.pzzz.vertx.worker.QueuedWorker;
import io.vertx.core.Closeable;
import io.vertx.core.Future;
import io.vertx.core.Promise;

public abstract class ProcessExecutionController<T extends ExecutableProcess<V,U>, U extends ItemProcess<V>, V extends Serializable, W>
        extends QueuedWorker<U,W> implements Closeable {
    private static final Logger LOG = Logger.getLogger(ProcessExecutionController.class.getName());

    private final T process;
    private final Startup startup;

    public ProcessExecutionController(final T process, final Startup startup) {
        super(process.getParallelRequests());
        this.process = process;
        this.startup = startup;
    }

    public Future<Void> deployWorkers() {
        Promise<Void> promise = Promise.promise();
        Future.all(getWorker().stream().map(controller -> controller.deployWorkers(startup)).toList())
                .onSuccess(res -> promise.complete())
                .onFailure(error -> {
                    LOG.log(Level.SEVERE, error.getMessage(), error);
                    promise.fail(error);
            });
        return promise.future();
    }

    @Override
    public void close(final Promise<Void> completion) {
        Future.all(getWorker().stream().map(DeployableWorker::undeployWorkers).toList())
                .onSuccess(res -> completion.complete())
                .onFailure(error -> {
                    LOG.log(Level.SEVERE, error.getMessage(), error);
                    completion.fail(error);
            });
    }

    protected abstract List<DeployableWorker> getWorker();

    protected T getProcess() {
        return process;
    }

    protected void handleJobFailure(final U job, final Throwable error, final Promise<?> promise) {
        LOG.log(Level.WARNING, "Failed job for " + job.getItem() + " with error " + error.getMessage(), error);
        job.setErrorState();
        job.addMessage(error.getMessage());
        promise.fail(error);
    }
}
