package de.pzzz.vertx.worker;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.logging.Level;
import java.util.logging.Logger;

import de.pzzz.vertx.Startup;
import io.vertx.core.Future;
import io.vertx.core.eventbus.Message;

public class FileSaveVerticle extends WorkerVerticle<SaveFile> {
    private static final Logger LOG = Logger.getLogger(FileSaveVerticle.class.getName());
    private static final String DEFAULT_OUTDIR = "out";

    private Path baseDir;

    @Override
    protected Future<Startup> doSetup(final Startup startup) {
		baseDir = Path.of(startup.getConfig().getString("file.basePath"), startup.getConfig().getString("file.out.dir", DEFAULT_OUTDIR));
        File baseDirFile = baseDir.toFile();
        if (!baseDirFile.isDirectory() || !baseDirFile.canWrite()) {
            return Future.failedFuture("Failed to access " + baseDir);
        }
        return Future.succeededFuture(startup);
    }

    @Override
    protected void handleMessage(final Message<SaveFile> message) {
        Path targetFile = Path.of(baseDir.toString(), message.body().getName());
        LOG.info("Saving file " + targetFile);
        try {
            Files.createDirectories(targetFile.getParent());
            Files.write(targetFile, message.body().getContent());
        } catch (IOException e) {
            LOG.log(Level.WARNING, e.getMessage(), e);;
            message.fail(500, e.getMessage());
            return;
        }
        message.reply(targetFile.toString());
    }

}
