package de.pzzz.vertx.worker;

import java.util.logging.Logger;

import de.pzzz.vertx.Startup;
import io.vertx.config.ConfigRetriever;
import io.vertx.core.AbstractVerticle;
import io.vertx.core.Future;
import io.vertx.core.Promise;
import io.vertx.core.eventbus.Message;
import io.vertx.core.eventbus.MessageConsumer;

public abstract class WorkerVerticle<T> extends AbstractVerticle {
	private static final Logger LOG = Logger.getLogger(WorkerVerticle.class.getName());

	@Override
	public void start(final Promise<Void> startPromise) throws Exception {
		long startTime = System.currentTimeMillis();

		initConfig().compose(this::doSetup)
				.compose(this::setupConsumer)
				.onComplete(result -> LOG.info(() -> "Started " + this.getClass().getName() + " in " + (System.currentTimeMillis() - startTime) + "ms"))
				.onComplete(startPromise);
	}

	protected abstract Future<Startup> doSetup(final Startup startup);

	protected abstract void handleMessage(final Message<T> message);

	private Future<Startup> initConfig() {
		ConfigRetriever configRetriever = ConfigRetriever.create(vertx);
		configRetriever.getConfig();
		return configRetriever.getConfig().map(Startup::new);
	}

	private Future<Void> setupConsumer(final Startup startup) {
		LOG.info("Setup consumer for bus address " + startup.getConfig().getString(WorkerController.WORKER_BUS_ADDRESS_CONFIG));
		MessageConsumer<T> consumer = vertx.eventBus()
				.consumer(startup.getConfig().getString(WorkerController.WORKER_BUS_ADDRESS_CONFIG));
		consumer.handler(this::handleMessage);
		return Future.succeededFuture();
	}

}
